/*
 * AtomDescriptor.hpp
 *
 *  Created on: Feb 5, 2010
 *      Author: crueger
 */

#ifndef ATOMDESCRIPTOR_HPP_
#define ATOMDESCRIPTOR_HPP_


#include <vector>
#include <map>
#include <boost/shared_ptr.hpp>
#include "World.hpp"

class atom;

class AtomDescriptor
{
public:

  typedef boost::shared_ptr<AtomDescriptor> desc_ptr;

  friend atom* World::getAtom(AtomDescriptor& descriptor);
  friend std::vector<atom*> World::getAllAtoms(AtomDescriptor& descriptor);

  AtomDescriptor();
  virtual ~AtomDescriptor();
  AtomDescriptor(const AtomDescriptor&);

  virtual bool predicate(std::pair<int,atom*>)=0;
  // clone is only used internally to generate operators
  virtual desc_ptr clone() const=0;

protected:
  virtual atom* find();
  virtual std::vector<atom*> findAll();
  std::map<int,atom*>& getAtoms();
};

class AtomAndDescriptor : public AtomDescriptor
{
  friend AtomAndDescriptor operator&&(const AtomDescriptor &lhs, const AtomDescriptor &rhs);
public:
  AtomAndDescriptor(const AtomAndDescriptor&);
  ~AtomAndDescriptor();

  virtual desc_ptr clone() const;
  virtual bool predicate(std::pair<int,atom*>);

private:
  desc_ptr lhs;
  desc_ptr rhs;

  // private constructor to enforce construction through operators
  AtomAndDescriptor(const AtomDescriptor &_lhs, const AtomDescriptor &_rhs);
};

class AtomOrDescriptor : public AtomDescriptor
{
  friend AtomOrDescriptor  operator||(const AtomDescriptor &lhs, const AtomDescriptor &rhs);
public:
  AtomOrDescriptor(const AtomOrDescriptor &);
  ~AtomOrDescriptor();

  virtual desc_ptr clone() const;
  virtual bool predicate(std::pair<int,atom*>);

private:
  desc_ptr lhs;
  desc_ptr rhs;

  // private constructor to enforce construction through operators
  AtomOrDescriptor(const AtomDescriptor &_lhs, const AtomDescriptor &_rhs);
};

class AtomNotDescriptor : public AtomDescriptor
{
  friend AtomNotDescriptor operator!(const AtomDescriptor &arg);
public:
  AtomNotDescriptor(const AtomNotDescriptor &);
  ~AtomNotDescriptor();

  virtual desc_ptr clone() const;
  virtual bool predicate(std::pair<int,atom*>);

private:
  desc_ptr arg;

  // protected constructor to enforce construction through operators
  AtomNotDescriptor(const AtomDescriptor &_arg);
};

// no true short circuit, but the test of the second descriptor wont be done
AtomAndDescriptor operator&&(const AtomDescriptor &lhs, const AtomDescriptor &rhs);
AtomOrDescriptor  operator||(const AtomDescriptor &lhs, const AtomDescriptor &rhs);
AtomNotDescriptor operator!(const AtomDescriptor &arg);

#endif /* ATOMDESCRIPTOR_HPP_ */
